/* Demo Java application that demonstrate the Direct Row Handling API to connect with a mySQL database using the mySQL JDBC Driver
** SQL Anywhere Studio MobiLink Synchronization 10.0.1
** Written by Joshua Savill
** Product Manager - Sybase iAnywhere
** December 4, 2007
*/

import ianywhere.ml.script.*;
import java.sql.*;
import java.util.*;
import java.io.*;
import java.lang.*;

public class mySQLSync
{
	public static final int AUTH_USER = 1000;
    public static final int UNAUTH_USER = 4000;

	//mySQL database connection string
	static String mysql_url = "jdbc:mysql://jsavill-mySQL:3306/sademo?user=root&password=sql";

	//Connection context for the ML server
	DBConnectionContext _conn_context;

	//Connection context for mySQL database
	Connection _sync_connection;
	Connection _mysql_conn;
	Timestamp _last;
	String _mluser;

	/* Creates connection context with the ML Server Direct Row API
	** Establish and allocates resources for the mySQL database connection
	*/
	public mySQLSync( DBConnectionContext cc )
		throws SQLException
	{
		try
		{
			_conn_context = cc;
			_sync_connection = _conn_context.getConnection();
		}
		catch ( SQLException e )
		{
			freeJDBCResources();
			throw e;
		}
		catch ( Error e )
		{
			freeJDBCResources();
			throw e;
		}

		try
		{
			Class.forName( "com.mysql.jdbc.Driver" ).newInstance();
		  	_mysql_conn = DriverManager.getConnection( mysql_url );

		  	if( _mysql_conn == null )
		  	{
			  	System.out.println("==== MySQL Connection Failed!");
		  	}
		  	else
		  	{
			  	System.out.println("==== MySQL Connection OK!");
		  	}
		}
		catch ( SQLException ex )
		{
			System.out.println("SQLException: " + ex.getMessage());
			System.out.println("SQLState: " + ex.getSQLState());
    		System.out.println("VendorError: " + ex.getErrorCode());
		}
		catch ( Exception ee )
		{
			ee.printStackTrace();
		}
	}

	protected void finalize()
		throws SQLException, Throwable
	{
		super.finalize();
		freeJDBCResources();
	}

	/* Frees the JDBC resources for the mySQL and API connection
	*/
	private void freeJDBCResources()
		throws SQLException
	{
		_conn_context = null;
		_sync_connection = null;
    }

    public void endConnection()
    	throws SQLException
	{
		freeJDBCResources();
    }

    /* Code for the begin_download connection script
    ** Sets the last_download timestamp the ml user that is synchronzing
    ** This is obtained from the generated synchronization stream
    */
    public void beginDownload( Timestamp last_download, String mlUser )
	{
		_last = last_download;
		_mluser = mlUser;
	}

	/* Code for the authenticate_user connection script
	** Simple authentication to ensure an ml user name and password are present (dbmlsync -mp option needs to be set)
	*/
	public void authenticateUser ( InOutInteger auth_status, String ml_offusername, String user_password )
		throws Exception
	{
		if( ml_offusername == null || user_password == null )
		{
			auth_status.setValue(UNAUTH_USER);
		}
		else
		{
				auth_status.setValue(AUTH_USER);
		}
    }

	/* Code for the handle_error conneciton script
	** Checks for certain errors during the synchronization
	*/
    public String handleError( ianywhere.ml.script.InOutInteger actionCode, int errorCode,
    	String errorMessage, String user, String table )
	{
	    if( errorCode == -196 || errorCode == -194 )
		{
		    actionCode.setValue( 1000 );
		}
		else
		{
		    actionCode.setValue( 3000 );
		}
		return( null );
    }

	/* Code for the handle_UploadData connection script
	** Handles the upload of the synchronization
	** A function call exists for the Insert, Update and Delete off all tables involved in synchronization
	** Makes use of the getUploadedTables API call to obtain the table names from the upload stream
	*/
	public void handleUpload( UploadData ud )
		throws SQLException, IOException
	{
		int i;

		UploadedTableData tables[] = ud.getUploadedTables();

		//Loop through all the tables performing Inserts and Updates
		for( i=0; i<tables.length; i+=1 )
		{
			String table_name = tables[i].getName();

			if( table_name.equals("Admin") )
			{
				handleUploadAdminInsert( tables[i].getInserts() );
				handleUploadAdminUpdate( tables[i].getUpdates() );
			}
			else if( table_name.equals("Parent") )
			{
				handleUploadParentInsert( tables[i].getInserts() );
				handleUploadParentUpdate( tables[i].getUpdates() );
			}
			else if( table_name.equals("Child") )
			{
				handleUploadChildInsert( tables[i].getInserts() );
				handleUploadChildUpdate( tables[i].getUpdates() );
			}
			else if( table_name.equals("Inventory") )
			{
				handleUploadInventoryInsert( tables[i].getInserts() );
				handleUploadInventoryUpdate( tables[i].getUpdates() );
			}
			else
			{
					// Throw SQLException table_not_found
			}
		}

		//Loop through all the tables performing Deletes
		for( i=tables.length-1; i>=0; i-=1 )
		{
			String table_name = tables[i].getName();

			if( table_name.equals("Admin") )
			{
				handleUploadAdminDelete( tables[i].getDeletes() );
			}
			else if( table_name.equals("Parent") )
			{
				handleUploadParentDelete( tables[i].getDeletes() );
			}
			else if( table_name.equals("Child") )
			{
				handleUploadChildDelete( tables[i].getDeletes() );
			}
			else if( table_name.equals("Inventory") )
			{
				handleUploadInventoryDelete( tables[i].getDeletes() );
			}
			else
			{
					// Throw SQLException table_not_found
			}
		}
	}

	/* Handles the upload inserts for the Admin table
	*/
	public void handleUploadAdminInsert( java.sql.ResultSet ins )
		throws SQLException, IOException
	{
		PreparedStatement ps1 = _mysql_conn.prepareStatement( "insert into Admin values ( ?, ?, DEFAULT )" );

		while( ins.next() )
		{
			ps1.setLong( 1, ins.getLong(1) );
			ps1.setString( 2, ins.getString(2) );
			ps1.executeUpdate();
		}
	}

	/* Handles the upload inserts for the Parent table
	*/
	public void handleUploadParentInsert( java.sql.ResultSet ins )
		throws SQLException, IOException
	{
		PreparedStatement ps2 = _mysql_conn.prepareStatement( "insert into Parent values ( ?, ?, ?, DEFAULT )" );

		while( ins.next() )
		{
			ps2.setLong( 1, ins.getLong(1) );
			ps2.setString( 2, ins.getString(2) );
			ps2.setString( 3, ins.getString(3) );
			ps2.executeUpdate();
		}
	}

	/* Handles the upload inserts for the Child table
	*/
	public void handleUploadChildInsert( java.sql.ResultSet ins )
		throws SQLException, IOException
	{
		PreparedStatement ps3 = _mysql_conn.prepareStatement( "insert into Child values ( ?, ?, ?, DEFAULT )" );

		while( ins.next() )
		{
			ps3.setLong( 1, ins.getLong(1) );
			ps3.setLong( 2, ins.getLong(2) );
			ps3.setString( 3, ins.getString(3) );
			ps3.executeUpdate();
		}
	}

	/* Handles the upload inserts for the Inventory table
	*/
	public void handleUploadInventoryInsert( java.sql.ResultSet ins )
			throws SQLException, IOException
	{
		PreparedStatement ps10 = _mysql_conn.prepareStatement( "insert into Inventory values ( ?, ?, DEFAULT )" );

		while( ins.next() )
		{
			ps10.setLong( 1, ins.getLong(1) );
			ps10.setInt( 2, ins.getInt(2) );
			ps10.executeUpdate();
		}
	}

	/* For Updates use old row values in the where clause as well to avoid conflicts
	** First one in wins in this setup
	** Until all the data is in sync again, only 1 row can be updated at a time
	*/

	/* Handles the upload updates for the Admin table
	** Conflict resolution: first one in wins
	*/
	public void handleUploadAdminUpdate( UpdateResultSet upd )
		throws SQLException, IOException
	{
		PreparedStatement ps4 = _mysql_conn.prepareStatement( "update Admin set data = ? where admin_id = ? and data = ? " );

		while( upd.next() )
		{
			upd.setNewRowValues();
			ps4.setString( 1, upd.getString(2) );
			ps4.setLong( 2, upd.getLong(1) );
			upd.setOldRowValues();
			ps4.setString( 3, upd.getString(2) );

			if( ps4.executeUpdate() == 0 )
			{
				System.out.println("Updated Row Ignored in table Admin because of conflict.");
				upd.setNewRowValues();
				System.out.println("  New Row Values : admin_id = " + upd.getLong(1) + ", data = " + upd.getString(2) );
				upd.setOldRowValues();
				System.out.println("  Old Row Values : admin_id = " + upd.getLong(1) + ", data = " + upd.getString(2) );
			}
		}
	}

	/* Handles the upload updates for the Parent table
	** Conflict resolution: first one in wins
	*/
	public void handleUploadParentUpdate( UpdateResultSet upd )
		throws SQLException, IOException
	{
		PreparedStatement ps5 = _mysql_conn.prepareStatement( "update Parent set user_id = ?, data = ? where parent_id = ? and user_id = ? and data = ? " );

		while( upd.next() )
		{
			upd.setNewRowValues();
			ps5.setString( 1, upd.getString(2) );
			ps5.setString( 2, upd.getString(3) );
			ps5.setLong( 3, upd.getLong(1) );
			upd.setOldRowValues();
			ps5.setString( 4, upd.getString(2) );
			ps5.setString( 5, upd.getString(3) );

			if( ps5.executeUpdate() == 0 )
			{
				System.out.println("Updated Row Ignored in table Parent because of conflict.");
				upd.setNewRowValues();
				System.out.println("  New Row Values : parent_id = " + upd.getLong(1) + ", user_id = " + upd.getString(2) + ", data = " + upd.getString(3) );
				upd.setOldRowValues();
				System.out.println("  Old Row Values : parent_id = " + upd.getLong(1) + ", user_id = " + upd.getString(2) + ", data = " + upd.getString(3) );
			}
		}
	}

	/* Handles the upload updates for the Child table
	** Conflict resolution: first one in wins
	*/
	public void handleUploadChildUpdate( UpdateResultSet upd )
		throws SQLException, IOException
	{
		PreparedStatement ps6 = _mysql_conn.prepareStatement( "update Child set parent_id = ?, data = ? where child_id = ? and parent_id = ? and data = ? " );

		while( upd.next() )
		{
			upd.setNewRowValues();
			ps6.setLong( 1, upd.getLong(2) );
			ps6.setString( 2, upd.getString(3) );
			ps6.setLong( 3, upd.getLong(1) );
			upd.setOldRowValues();
			ps6.setLong( 4, upd.getLong(2) );
			ps6.setString( 5, upd.getString(3) );

			if( ps6.executeUpdate() == 0 )
			{
				System.out.println("Updated Row Ignored in table Child because of conflict.");
				upd.setNewRowValues();
				System.out.println("  New Row Values : child_id = " + upd.getLong(1) + ", parent_id = " + upd.getLong(2) + ", data = " + upd.getString(3) );
				upd.setOldRowValues();
				System.out.println("  Old Row Values : child_id = " + upd.getLong(1) + ", parent_id = " + upd.getLong(2) + ", data = " + upd.getString(3) );
			}
		}
	}

	/* Handles the upload updates for the Inventory table
	** Conflict resolution: updated inventory data column is recomputed
	*/
	public void handleUploadInventoryUpdate( UpdateResultSet upd )
		throws SQLException, IOException
	{
		PreparedStatement ps11 = _mysql_conn.prepareStatement( "update Inventory set data = ? where inventory_id = ? " );

		while( upd.next() )
		{
			int inventory = 0;
			int inventoryRes = 0;
			int pkValue = upd.getInt(1);
			int centralValue = getMyCentralData(pkValue);


			upd.setNewRowValues();
			int newRowValue = upd.getInt(2);
			upd.setOldRowValues();
			int oldRowValue = upd.getInt(2);

			// Demostrate as if an order exists at the remote site
			if( oldRowValue != centralValue )
			{
				// Handling detected conflict
				inventoryRes = centralValue - newRowValue;
				System.out.println("centralValue = " + centralValue);
				System.out.println("inventory = " + inventory);
				System.out.println("inventoryRes = " + inventoryRes);
			}
			else
			{
				inventoryRes = oldRowValue - newRowValue;
			}

			// Required if the inventory exist for the remote site
			// inventory = oldRowValue - newRowValue;
			// inventoryRes = centralValue - inventory;

			ps11.setInt( 1, inventoryRes );
			ps11.setLong( 2, upd.getLong(1) );

			if( ps11.executeUpdate() != 0 )
			{
				upd.setNewRowValues();
				System.out.println("New Row Value: data = " + upd.getInt(2) );
				upd.setOldRowValues();
				System.out.println("Old Row Values : data = " + upd.getInt(2) );
				System.out.println("New value computed for Inventory: " + inventoryRes );
			}
			else if( ps11.executeUpdate() == 0 )
			{
				System.out.println("Updated Row Ignored in table Inventory because of conflict.");
				upd.setNewRowValues();
				System.out.println("  New Row Values : inventory_id = " + upd.getLong(1) + ", data = " + upd.getString(2) );
				upd.setOldRowValues();
				System.out.println("  Old Row Values : inventory_id = " + upd.getLong(1) + ", data = " + upd.getString(2) );
			}
		}
	}

	/* Queries the consolidated for the actual inventory value of the upload column
	*/
	public int getMyCentralData( int pkValue )
		throws SQLException, IOException
	{
		PreparedStatement ps13 = _mysql_conn.prepareStatement( "select data from Inventory where inventory_id = ? ");
		ps13.setInt( 1, pkValue );
		ResultSet rs = ps13.executeQuery();

		int value = 0;

		while( rs.next() )
		{
			value = rs.getInt(1);
		}

		return value;
	}

	/* Handles the upload deletes for the Admin table
	*/
	public void handleUploadAdminDelete( java.sql.ResultSet del )
		throws SQLException, IOException
	{
		PreparedStatement ps7 = _mysql_conn.prepareStatement( "delete from Admin where admin_id = ?" );

		while( del.next() )
		{
			ps7.setLong( 1, del.getLong(1) );

			if( ps7.executeUpdate() == 0 )
			{
				System.out.println("Deleted Row Ignored in table Admin.");
				System.out.println("  admin_id = " + del.getLong(1) );
			}
		}
	}

	/* Handles the upload deletes for the Parent table
	*/
	public void handleUploadParentDelete( java.sql.ResultSet del )
		throws SQLException, IOException
	{
		PreparedStatement ps8 = _mysql_conn.prepareStatement( "delete from Parent where parent_id = ?" );

		while( del.next() )
		{
			ps8.setLong( 1, del.getLong(1) );

			if( ps8.executeUpdate() == 0 )
			{
				System.out.println("Deleted Row Ignored in table Parent.");
				System.out.println("  parent_id = " + del.getLong(1) );
			}
		}
	}

	/* Hanldes the upload deletes for the Child table
	*/
	public void handleUploadChildDelete( java.sql.ResultSet del )
		throws SQLException, IOException
	{
		PreparedStatement ps9 = _mysql_conn.prepareStatement( "delete from Child where child_id = ?" );

		while( del.next() )
		{
			ps9.setLong( 1, del.getLong(1) );

			if( ps9.executeUpdate() == 0 )
			{
				System.out.println("Deleted Row Ignored in table Child.");
				System.out.println("  child_id = " + del.getLong(1) );
			}
		}
	}

	/* Handles the upload deletes for the Inventory table
	*/
	public void handleUploadInventoryDelete( java.sql.ResultSet del )
			throws SQLException, IOException
	{
		PreparedStatement ps12 = _mysql_conn.prepareStatement( "delete from Inventory where inventory_id = ?" );

		while( del.next() )
		{
			ps12.setLong( 1, del.getLong(1) );

			if( ps12.executeUpdate() == 0 )
			{
				System.out.println("Deleted Row Ignored in table Inventory.");
				System.out.println("  inventory_id = " + del.getLong(1) );
			}
		}
	}

	/* Handles the download events for all table involved in synchronization
	** All Deletes are handled in the Track_Deletes table
	** The insert and updates are based on the last_modified column that is updated by trigger during column modification
	** Update conflict resolution: last modification to the column wins
	*/
	public void handleDownload()
		throws SQLException
	{
		DownloadData dd = _conn_context.getDownloadData();

		//Download new data for the Admin table
		DownloadTableData dtdAdmin = dd.getDownloadTableByName("Admin");

		PreparedStatement downCurAdmin = dtdAdmin.getUpsertPreparedStatement();
		PreparedStatement dataAdmin = _mysql_conn.prepareStatement( "select admin_id, data from Admin where last_modified >= ?" );
		dataAdmin.setTimestamp( 1, _last );
		ResultSet rsAdmin = dataAdmin.executeQuery();

		while( rsAdmin.next() )
		{
			downCurAdmin.setLong( 1, rsAdmin.getLong(1) );
			downCurAdmin.setString( 2, rsAdmin.getString(2) );

			if( downCurAdmin.executeUpdate() == 0 )
			{
				System.out.println("Download Row Filtered from table Admin.");
				System.out.println("  admin_id = " + rsAdmin.getLong(1) + ", data = " + rsAdmin.getString(2) );
			}
		}

		downCurAdmin.close();
		rsAdmin.close();

		//Download deleted data for the Admin table
		PreparedStatement downDelCurAdmin = dtdAdmin.getDeletePreparedStatement();
		PreparedStatement dataDelAdmin = _mysql_conn.prepareStatement( "select id from Track_Deletes where table_name = 'Admin' and time_deleted >= ?" );
		dataDelAdmin.setTimestamp( 1, _last );
		ResultSet rsDelAdmin = dataDelAdmin.executeQuery();

		while( rsDelAdmin.next() )
		{
			downDelCurAdmin.setLong( 1, rsDelAdmin.getLong(1) );

			if( downDelCurAdmin.executeUpdate() == 0 )
			{
				System.out.println("Download Delete Row Filtered from table Admin.");
				System.out.println("  admin_id = " + rsDelAdmin.getLong(1) );
			}
		}

		downDelCurAdmin.close();
		rsDelAdmin.close();

		//Download new data for the Parent table
		DownloadTableData dtdParent = dd.getDownloadTableByName("Parent");

		PreparedStatement downCurParent = dtdParent.getUpsertPreparedStatement();
		PreparedStatement dataParent = _mysql_conn.prepareStatement( "select parent_id, user_id, data from Parent where user_id = ? and last_modified >= ?" );
		dataParent.setString( 1, _mluser );
		dataParent.setTimestamp( 2, _last );
		ResultSet rsParent = dataParent.executeQuery();

		while( rsParent.next() )
		{
			downCurParent.setLong( 1, rsParent.getLong(1) );
			downCurParent.setString( 2, rsParent.getString(2) );
			downCurParent.setString( 3, rsParent.getString(3) );

			if( downCurParent.executeUpdate() == 0 )
			{
				System.out.println("Download Row Filtered from table Parent.");
				System.out.println("  parent_id = " + rsParent.getLong(1) + ", user_id = " + rsParent.getString(2) + ", data = " + rsParent.getString(3) );
			}
		}

		downCurParent.close();
		rsParent.close();

		//Download deleted data for the Parent table
		PreparedStatement downDelCurParent = dtdParent.getDeletePreparedStatement();
		PreparedStatement dataDelParent = _mysql_conn.prepareStatement( "select id from Track_Deletes where table_name = 'Parent' and time_deleted >= ?" );
		dataDelParent.setTimestamp( 1, _last );
		ResultSet rsDelParent = dataDelParent.executeQuery();

		while( rsDelParent.next() )
		{
			downDelCurParent.setLong( 1, rsDelParent.getLong(1) );

			if( downDelCurParent.executeUpdate() == 0 )
			{
				System.out.println("Download Delete Row Filtered from table Parent.");
				System.out.println("  parent_id = " + rsDelParent.getLong(1) );
			}
		}

		downDelCurParent.close();
		rsDelParent.close();

		//Download new data for the Child table
		DownloadTableData dtdChild = dd.getDownloadTableByName("Child");

		PreparedStatement downCurChild = dtdChild.getUpsertPreparedStatement();
		PreparedStatement dataChild = _mysql_conn.prepareStatement( "select Child.child_id, Child.parent_id, Child.data from Child, Parent where Parent.parent_id = Child.parent_id and Parent.user_id = ? and Child.last_modified >= ?" );
		dataChild.setString( 1, _mluser );
		dataChild.setTimestamp( 2, _last );
		ResultSet rsChild = dataChild.executeQuery();

		while( rsChild.next() )
		{
			downCurChild.setLong( 1, rsChild.getLong(1) );
			downCurChild.setLong( 2, rsChild.getLong(2) );
			downCurChild.setString( 3, rsChild.getString(3) );

			if( downCurChild.executeUpdate() == 0 )
			{
				System.out.println("Download Row Filtered from table Child.");
				System.out.println("  child_id = " + rsChild.getLong(1) + ", parent_id = " + rsChild.getLong(2) + ", data = " + rsChild.getString(3) );
			}
		}

		downCurChild.close();
		rsChild.close();

		//Download deleted data for the Child table
		PreparedStatement downDelCurChild = dtdChild.getDeletePreparedStatement();
		PreparedStatement dataDelChild = _mysql_conn.prepareStatement( "select id from Track_Deletes where table_name = 'Child' and time_deleted >= ?" );
		dataDelChild.setTimestamp( 1, _last );
		ResultSet rsDelChild = dataDelChild.executeQuery();

		while( rsDelChild.next() )
		{
			downDelCurChild.setLong( 1, rsDelChild.getLong(1) );

			if( downDelCurChild.executeUpdate() == 0 )
			{
				System.out.println("Download Delete Row Filtered from table Child.");
				System.out.println("  child_id = " + rsDelChild.getLong(1) );
			}
		}

		downDelCurChild.close();
		rsDelChild.close();

		//Download new data from the Inventory table
		DownloadTableData dtdInventory = dd.getDownloadTableByName("Inventory");

		PreparedStatement downCurInventory = dtdInventory.getUpsertPreparedStatement();
		PreparedStatement dataInventory = _mysql_conn.prepareStatement( "select inventory_id, data from Inventory where last_modified >= ?" );
		dataInventory.setTimestamp( 1, _last );
		ResultSet rsInventory = dataInventory.executeQuery();

		while( rsInventory.next() )
		{
			downCurInventory.setLong( 1, rsInventory.getLong(1) );
			downCurInventory.setInt( 2, rsInventory.getInt(2) );

			if( downCurInventory.executeUpdate() == 0 )
			{
				System.out.println("Download Row Filtered from table Inventory.");
				System.out.println("  inventory_id = " + rsInventory.getLong(1) + ", data = " + rsInventory.getInt(2) );
			}
		}

		downCurInventory.close();
		rsInventory.close();

		//Download deleted data from the Inventory table
		PreparedStatement downDelCurInventory = dtdInventory.getDeletePreparedStatement();
		PreparedStatement dataDelInventory = _mysql_conn.prepareStatement( "select id from Track_Deletes where table_name = 'Inventory' and time_deleted >= ?" );
		dataDelInventory.setTimestamp( 1, _last );
		ResultSet rsDelInventory = dataDelInventory.executeQuery();

		while( rsDelInventory.next() )
		{
			downDelCurInventory.setLong( 1, rsDelInventory.getLong(1) );

			if( downDelCurInventory.executeUpdate() == 0 )
			{
				System.out.println("Download Delete Row Filtered from table Inventory.");
				System.out.println("  inventory_id = " + rsDelAdmin.getLong(1) );
			}
		}

		downDelCurInventory.close();
		rsDelInventory.close();
	}
}

