-- Demo Java application that demonstrate the Direct Row Handling API to connect with a mySQL database using the mySQL JDBC Driver
-- SQL Anywhere Studio MobiLink Synchronization 10.0.1
-- Written by Joshua Savill
-- Product Manager - Sybase iAnywhere
-- December 4, 2007

--
-- Create the tables to be involved in synchronization
--

create table Admin (
  admin_id      bigint not null auto_increment primary key,
  data          varchar(30),
  last_modified datetime not null default '1900/01/01 00:00:00'
);

create table Parent (
  parent_id     bigint not null auto_increment primary key,
  user_id       varchar(128),
  data          varchar(30),
  last_modified datetime not null default '1900/01/01 00:00:00'
);

create table Child (
  child_id      bigint not null auto_increment primary key,
  parent_id     bigint references Parent,
  data          varchar( 30 ),
  last_modified datetime not null default '1900/01/01 00:00:00'
);

create table Inventory (
  inventory_id      bigint not null auto_increment primary key,
  data          int not null,
  last_modified datetime not null default '1900/01/01 00:00:00'
);

--
-- The Track_Deletes table will hold deletes for all tables
-- This is done for the purpose of making this demo more simple
-- For a production setup, you would want a Track_Deletes table
-- specific for each base table
--

create table Track_Deletes (
  pkey         bigint not null auto_increment primary key,
  table_name   varchar(128) not null,
  id           bigint not null,
  time_deleted datetime not null
);


--
-- Create the necessary triggers for synchronization and data
-- management
--

DELIMITER |

CREATE TRIGGER bi_admin BEFORE INSERT ON Admin FOR EACH ROW
BEGIN
  SET NEW.last_modified = now();
END;
|

CREATE TRIGGER bu_admin BEFORE UPDATE ON Admin FOR EACH ROW
BEGIN
  SET NEW.last_modified = now();
END;
|

CREATE TRIGGER bd_admin BEFORE DELETE ON Admin FOR EACH ROW
BEGIN
  INSERT INTO Track_Deletes (table_name, id, time_deleted) VALUES ( 'Admin', OLD.admin_id, now() );
END;
|

CREATE TRIGGER bi_parent BEFORE INSERT ON Parent FOR EACH ROW
BEGIN
  SET NEW.last_modified = now();
END;
|

CREATE TRIGGER bu_parent BEFORE UPDATE ON Parent FOR EACH ROW
BEGIN
  SET NEW.last_modified = now();
END;
|

CREATE TRIGGER bd_parent BEFORE DELETE ON Parent FOR EACH ROW
BEGIN
  INSERT INTO Track_Deletes (table_name, id, time_deleted) VALUES ( 'Parent', OLD.parent_id, now() );
END;
|

CREATE TRIGGER bi_child BEFORE INSERT ON Child FOR EACH ROW
BEGIN
  SET NEW.last_modified = now();
END;
|

CREATE TRIGGER bu_child BEFORE UPDATE ON Child FOR EACH ROW
BEGIN
  SET NEW.last_modified = now();
END;
|

CREATE TRIGGER bd_child BEFORE DELETE ON Child FOR EACH ROW
BEGIN
  INSERT INTO Track_Deletes (table_name, id, time_deleted) VALUES ( 'Child', OLD.child_id, now() );
END;
|

CREATE TRIGGER bi_inventory BEFORE INSERT ON Inventory FOR EACH ROW
BEGIN
  SET NEW.last_modified = now();
END;
|

CREATE TRIGGER bu_inventory BEFORE UPDATE ON Inventory FOR EACH ROW
BEGIN
  SET NEW.last_modified = now();
END;
|

CREATE TRIGGER bd_inventory BEFORE DELETE ON Inventory FOR EACH ROW
BEGIN
  INSERT INTO Track_Deletes (table_name, id, time_deleted) VALUES ( 'Inventory', OLD.inventory_id, now() );
END;
|

DELIMITER ;