-- Demo Java application that demonstrate the Direct Row Handling API to connect with a mySQL database using the mySQL JDBC Driver
-- SQL Anywhere Studio MobiLink Synchronization 10.0.1
-- Written by Joshua Savill
-- Product Manager - Sybase iAnywhere
-- December 4, 2007

/*------------------------------------------------------------------------------
*                           Set some Database Options.
*-----------------------------------------------------------------------------*/
set option public.Global_database_id = 100;
set option public.delete_old_logs = 'ON';

/*------------------------------------------------------------------------------
*                           Create table 'DBA.Admin'.
*-----------------------------------------------------------------------------*/

create table Admin (
  admin_id  bigint default global autoincrement ( 1000 ) primary key,
  data  varchar(30)
);

/*------------------------------------------------------------------------------
*                          Create table 'DBA.Parent'.
*-----------------------------------------------------------------------------*/

create table Parent (
  parent_id  bigint default global autoincrement ( 1000 ) primary key,
  user_id  varchar(128),
  data  varchar(30)
);

/*------------------------------------------------------------------------------
*                           Create table 'DBA.Child'.
*-----------------------------------------------------------------------------*/

create table Child (
  child_id bigint default global autoincrement ( 1000 ) primary key,
  parent_id bigint,
  data varchar( 30 ),
  foreign key ( parent_id )
  references parent ( parent_id )
  on delete cascade
);

/*------------------------------------------------------------------------------
*                         Create table 'DBA.Inventory'.
*-----------------------------------------------------------------------------*/

create table Inventory (
  inventory_id  bigint default global autoincrement ( 1000 ) primary key,
  data  int
);

/*------------------------------------------------------------------------------
*                          Procedures to add test data.
*-----------------------------------------------------------------------------*/

create procedure DBA.rep_data(in iter integer default 1,in do_commit integer default 0)
begin
  declare loop_var integer;
  
  set loop_var=0;
  while loop_var < iter loop
    insert into Admin(data) values(cast(RAND(*)*20+1 as char(10)));
    insert into Parent(user_id,data) values('rem100',cast(RAND(*)*20+1 as char(10)));
    insert into Child(parent_id,data) values(@@identity,cast(RAND(*)*20+1 as char(10)));
    set loop_var=loop_var+1;
    if do_commit = 1 then
      commit;
    end if
  end loop;
end;

create procedure DBA.rep_data_inventory(in iter integer default 1,in do_commit integer default 0)
begin
  declare loop_var integer;
  
  set loop_var=0;
  while loop_var < iter loop
    insert into Inventory(data) values(100);
    set loop_var=loop_var+1;
    if do_commit = 1 then
      commit;
    end if
  end loop;
end;

/*------------------------------------------------------------------------------
*                           Create publication 'p1'.
*-----------------------------------------------------------------------------*/

CREATE PUBLICATION p1 ( TABLE Admin, TABLE Parent, TABLE Child, TABLE Inventory );

/*------------------------------------------------------------------------------
*                          Create the user 'rem100'.
*-----------------------------------------------------------------------------*/

CREATE SYNCHRONIZATION USER "rem100";

/*------------------------------------------------------------------------------
*                  Create subscription to 'p1' for 'rem100'.
*-----------------------------------------------------------------------------*/

CREATE SYNCHRONIZATION SUBSCRIPTION TO p1 FOR "rem100" TYPE 'TCPIP' ADDRESS 'host=jsavill-d620;port=2439' OPTION SV='v1';
